#ifndef IMAGE_H                /* image.h */
#define IMAGE_H

#define VR_MAGIC 0x59a66a90

typedef unsigned char octet;

/* ---------------------- Structures de donnees Image -------------------- */

struct _Image {

  int   magicNumber; /* Pour verifier que on a (pas) alloue de la memoire */

  int   nbRow;
  int   nbCol;

  octet *ptrZone;        /* Le "vrai" pointeur sur la zone image          */
  octet **zone;          /* Un pointeur qui permet : zone[ligne][colonne] */

};

typedef struct _Image Image;

/* --------------------- Structures de donnees ImageRVB ------------------ */

struct _ImageRVB {

  int   magicNumber; /* Pour verifier que on a (pas) alloue de la memoire */

  int   nbRow;
  int   nbCol;

  Image R;
  Image V;
  Image B;

};

typedef struct _ImageRVB ImageRVB;

/* ------------------- Prototypes des fonctions sur Image ---------------- */

void mallocImage(Image *out, int nbRow, int nbCol);
void freeImage(Image *in);

void loadImage(Image *out, const char *nomFichier);
void saveImage(const Image *in, const char *nomFichier);

                                             /* Appel d'un utilitaire,     */
void displayImage(const char *nomFichier);   /* voir au debut de image.c : */
                                        /* static char visualiseur[]="xv"; */

void setImage(Image *out, octet val); /* Tous les pixels initialises a val */
void lineImage(Image *inout,int x1,int y1,int x2,int y2,octet val);

void copyImage(const Image *in, Image *out); /* Copie de in dans out       */

void ImagetoImageRVB(const Image *in, ImageRVB *out);  /* R=in, V=in, B=in */

void writePix(Image *out, int ligne, int colonne, octet val);
void readPix(const Image *in, int ligne, int colonne, octet *val);

/* ------------------ Prototypes des fonctions sur ImageRVB -------------- */

void mallocImageRVB(ImageRVB *out, int nbRow, int nbCol);
void freeImageRVB(ImageRVB *in);

void loadImageRVB(ImageRVB *out, const char *nomFichier);
void saveImageRVB(const ImageRVB *in, const char *nomFichier);

                                             /* Appel d'un utilitaire,     */
void displayImageRVB(const char *nomFichier);/* voir au debut de image.c : */
                                        /* static char visualiseur[]="xv"; */

void setImageRVB(ImageRVB *out, octet valR,  /* Tous les pixels initialises */
                                octet valV,  /* a valR,valV,valB            */
                                octet valB);
void lineImageRVB(ImageRVB *inout,int x1,int y1,int x2,int y2, octet valR,
                                                               octet valV,
                                                               octet valB);

void copyImageRVB(const ImageRVB *in, ImageRVB *out);/* Copie de in dans out */

void ImageRVBtoImage(const ImageRVB *in, Image *out); /* (R+V+B)/3 */

void writePixRVB(ImageRVB *out, int ligne, int colonne, octet valR,
                                                        octet valV,
                                                        octet valB);
void readPixRVB(const ImageRVB *in, int ligne, int colonne, octet *valR,
                                                            octet *valV,
                                                            octet *valB);
#endif /* IMAGE_H */

/* -------------------------------- Remarques ---------------------------- */
/*
  --- Remarque 1 --------------------------------------------------------

  mallocImage    : si necessaire (i.e. pas alloue ou pas de la bonne taile):
                   - freeImage et
                   - malloc.
  mallocImageRVB : 3 mallocImage

  freeImage      : si necessaire (i.e. alloue) :
                   - free de ptrZone et
                   - free de zone.
  freeImageRVB   : 3 freeImage

  loadImage      : mallocImage avant le chargement
  loadImageRVB   : mallocImageRVB avant le chargement

  writePix       : On peut faire aussi zone[ligne][colonne]
  readPix        : On peut faire aussi zone[ligne][colonne]

  writePixRVB    : On peut faire aussi zoneR[ligne][colonne]
                                       zoneV[ligne][colonne]
                                       zoneB[ligne][colonne]
  readPixRVB     : On peut faire aussi zoneR[ligne][colonne]
                                       zoneV[ligne][colonne]
                                       zoneB[ligne][colonne]

  --- Remarque 2 --------------------------------------------------------

   void loadImage(const char *nomFichier, Image *out);
   void loadImageRVB(const char *nomFichier, Image *out);
   void saveImage(const Image *in, const char *nomFichier)
   void saveImageRVB(const Image *in, const char *nomFichier)

  Pour les fichiers "image" : .ras, .ppm, .pgm
  -------------------------

  -----------------------------------------------------------------
*/
