#include "image.h"                 /* image.c */
#include "LoadSaveImages.h"

                   /**************************************/
                       static char visualiseur[]="xv";
                   /**************************************/

#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

#define absT(x) (((x)>0)?(x):(-(x)))

/* ------------------ Les fonctions de image.h/image.c ------------------ */
/*  - mallocImage, freeImage,                                             */
/*  - loadImage, saveImage,                                               */ 
/*  - displayImage,                                                       */
/*  - setImage, lineImage                                                 */
/*  - copyImage,                                                          */
/*  - writePix, readPix.                                                  */
/* ---------------------------------------------------------------------- */

/* -- mallocImage -- */
void mallocImage(Image *out, int nbRow, int nbCol)
{
 int i=0;

 if  (out->magicNumber==VR_MAGIC)
 {
  if (out->nbRow==nbRow && out->nbCol==nbCol) return;
 }

 freeImage(out);

 out->magicNumber=VR_MAGIC;

 out->ptrZone = (octet*)malloc(sizeof(octet)*nbRow*nbCol);
 if (out->ptrZone==NULL) { fprintf(stderr,"Pb, mallocImage::new (1)\n");
                           exit(1);
 }
 
 out->zone   = (octet**)malloc(sizeof(octet*)*nbRow);
 if (out->zone==NULL)    { freeImage(out);
                           fprintf(stderr,"Pb, mallocImage::new (2)\n");
                           exit(1);
 }

 for(i=0;i<nbRow;i++)
 {
  out->zone[i]=out->ptrZone+i*nbCol;
 }

 out->nbRow=nbRow;
 out->nbCol=nbCol;
}

/* -- freeImage -- */
void freeImage(Image *in)
{
 if (in->magicNumber==VR_MAGIC)
 {
  if (in->zone!=NULL)    free(in->zone);
  if (in->ptrZone!=NULL) free(in->ptrZone);
 }
 
 in->magicNumber=0;
 in->nbRow=0;
 in->nbCol=0;
}

/* -- loadImage -- */
void loadImage(Image *out, const char *nomFichier)
{
 loadingImage(out,nomFichier);
}

/* -- saveImage -- */
void saveImage(const Image *in, const char *nomFichier)
{
 savingImage(in,nomFichier);
}

/* -- displayImage -- */
void displayImage(const char *nomFichier)
{
 struct stat fileStat;
 pid_t pid;

 if ((stat(nomFichier,&fileStat)==-1) ||
     (!S_ISREG(fileStat.st_mode))     ||
     (access(nomFichier,R_OK)==-1))      /* Pb pour lire le fichier */
 {
  fprintf(stderr,"displayImage: Can't read %s file\n",nomFichier);
  return;
 }

 do {
       pid = fork();
 } while (pid==-1 && errno==EAGAIN);

 if (pid==-1)
 {
   fprintf(stderr,"displayImage: Can't creat a new process (fork)\n");
   return;
 }

 if (pid==0)  /* Le fils seulement */
 {

  execlp(visualiseur,visualiseur,nomFichier,NULL);

  if (!strcmp(visualiseur,"xv"))
  {
   fprintf(stderr,"\nImpossible de trouver xv dans le PATH\n");
   fprintf(stderr,"=>Il faut executer la commande Shell suvante :\n");
   fprintf(stderr,"export PATH=...\n");
  }
  else {
   fprintf(stderr,
           "\nImpossible de trouver le visualiseur d'images %s "
           "dans le PATH\n",visualiseur);
  }

  exit(1);
 }
}

/* -- setImage -- */
void setImage(Image *out,octet val)
{
  memset(out->ptrZone,val,out->nbRow*out->nbCol);
}

/* -- lineImage -- */
void lineImage(Image *inout,int x1,int y1,int x2,int y2,octet val)
{
 int dx,dy,sx,sy,x,y;
 double a,b;
 
 dx=x2-x1; dy=y2-y1;
 if (dx > 0) sx = 1; else sx = -1;
 if (dy > 0) sy = 1; else sy = -1;

 if (dx == 0)
    if (y1 > y2) for(y=y2;y<=y1;y++) inout->zone[x1][y]=val;
            else for(y=y1;y<=y2;y++) inout->zone[x1][y]=val;
 else
 if (dy == 0)
    if (x1 > x2) for(x=x2;x<=x1;x++) inout->zone[x][y1]=val;
            else for(x=x1;x<=x2;x++) inout->zone[x][y1]=val;
 else
 if (absT(dy) < absT(dx))
     {
      a=(double)dy / (double) dx;
      b=(double)y1 - a*(double)x1;
      x=x1;
      while (x != x2)
          {
           inout->zone[x][(int)(a*(double)x+b)]=val;
           if (sx > 0) x++; else x--;
          }
      inout->zone[x2][y2]=val;
     }
     else
     {
      a=(double)dx / (double) dy;
      b=(double)x1 - a*(double)y1;
      y=y1;
      while (y != y2)
          {
           inout->zone[(int)(a*(double)y+b)][y]=val;
           if (sy > 0) y++; else y--;
          }
      inout->zone[x2][y2]=val;
     }
}

/* -- copyImage -- */
void copyImage(const Image *in, Image *out)
{
 int nbRowI=in->nbRow, nbColI=in->nbCol;

 mallocImage(out,nbRowI,nbColI); /* Si necessaire */

 memcpy(out->ptrZone,in->ptrZone,sizeof(octet)*nbRowI*nbColI);
}

void ImagetoImageRVB(const Image *in, ImageRVB *out)  /* R=in, V=in, B=in */
{
 int nbRowI=in->nbRow, nbColI=in->nbCol;
 int l,c;

 mallocImageRVB(out,nbRowI,nbColI); /* Si necessaire */

 for(l=0;l<nbRowI;l++)
  for(c=0;c<nbColI;c++)
   {
    out->R.zone[l][c]=in->zone[l][c];
    out->V.zone[l][c]=in->zone[l][c];
    out->B.zone[l][c]=in->zone[l][c];
   }
}

/* -- writePix -- */
void writePix(Image *out, int ligne, int colonne, octet val)
{
 out->zone[ligne][colonne]=val;
}

/* -- readPix -- */
void readPix(const Image *in, int ligne, int colonne, octet *val)
{
 *val=in->zone[ligne][colonne];
}


/* ------------------ Les fonctions de image.h/image.c ------------------ */
/*  - mallocImageRVB, freeImageRVB,                                       */
/*  - loadImageRVB, saveImageRVB,                                         */ 
/*  - displayImageRVB,                                                    */
/*  - setImageRVB, lineImageRVB                                           */
/*  - copyImageRVB,                                                       */
/*  - writePixRVB, readPixRVB.                                            */
/* ---------------------------------------------------------------------- */

/* ------------- Code des fonctions de image.h/image.c ------------------ */

/* -- mallocImageRVB -- */
void mallocImageRVB(ImageRVB *out, int nbRow, int nbCol)
{
 mallocImage(&(out->R),nbRow,nbCol);
 mallocImage(&(out->V),nbRow,nbCol);
 mallocImage(&(out->B),nbRow,nbCol);

 out->magicNumber=VR_MAGIC;

 out->nbRow=nbRow;
 out->nbCol=nbCol;
}

/* -- freeImageRVB -- */
void freeImageRVB(ImageRVB *in)
{
 freeImage(&in->R);
 freeImage(&in->V);
 freeImage(&in->B);
 
 in->magicNumber=0;

 in->nbRow=0;
 in->nbCol=0;
}

/* -- loadImageRVB -- */
void loadImageRVB(ImageRVB *out, const char *nomFichier)
{
 loadingImageRVB(out,nomFichier);
}

/* -- saveImageRVB -- */
void saveImageRVB(const ImageRVB *in, const char *nomFichier)
{
 savingImageRVB(in,nomFichier);
}

/* -- displayImageRVB -- */
void displayImageRVB(const char *nomFichier)
{
 displayImage(nomFichier); 
}

/* -- setImageRVB -- */
void setImageRVB(ImageRVB *out,octet valR,octet valB,octet valV)
{
  setImage(&out->R,valR);
  setImage(&out->V,valV);
  setImage(&out->B,valB);
}

/* -- lineImageRVB -- */
void lineImageRVB(ImageRVB *inout,int x1,int y1,int x2,int y2,octet valR,
                                                              octet valV,
                                                              octet valB)
{
 lineImage(&inout->R,x1,y1,x2,y2,valR);
 lineImage(&inout->V,x1,y1,x2,y2,valV);
 lineImage(&inout->B,x1,y1,x2,y2,valB);
}

/* -- copyImageRVB -- */
void copyImageRVB(const ImageRVB *in, ImageRVB *out)
{
 int nbRowI=in->nbRow, nbColI=in->nbCol;

 mallocImageRVB(out,nbRowI,nbColI); /* Si necessaire */

 copyImage(&in->R,&out->R);
 copyImage(&in->V,&out->V);
 copyImage(&in->B,&out->B);
}

void ImageRVBtoImage(const ImageRVB *in, Image *out)  /* (R+V+B)/3 */
{
 int nbRowI=in->nbRow, nbColI=in->nbCol;
 int l,c;

 mallocImage(out,nbRowI,nbColI); /* Si necessaire */

 for(l=0;l<nbRowI;l++)
  for(c=0;c<nbColI;c++)
   { 
    int red,green,blue;

    red   = in->R.zone[l][c];
    green = in->V.zone[l][c];
    blue  = in->B.zone[l][c];
    out->zone[l][c]=(octet)((red+green+blue)/3);
   }
}

/* -- writePixRVB -- */
void writePixRVB(ImageRVB *out, int ligne, int colonne, octet valR,
                                                        octet valV,
                                                        octet valB)
{
 writePix(&out->R,ligne,colonne,valR);
 writePix(&out->V,ligne,colonne,valV);
 writePix(&out->B,ligne,colonne,valB);
}

/* -- readPixRVB -- */
void readPixRVB(const ImageRVB *in, int ligne, int colonne, octet *valR,
                                                            octet *valV,
                                                            octet *valB)
{
 readPix(&in->R,ligne,colonne,valR);
 readPix(&in->V,ligne,colonne,valV);
 readPix(&in->B,ligne,colonne,valB);
}
